//**************************************************************************//
//* FRIDA: flexible rapid interactive data analysis                        *//
//* frida2.cpp: main loop                                                  *//
//* (C) Joachim Wuttke 1990-, C++ rewrite 2001-                            *//
//* http://www.messen-und-deuten.de/frida                                  *//
//**************************************************************************//

#include <string.h>
#include <stdlib.h>
#include <iostream>
#include <gsl/gsl_errno.h>

#include <ask_simple_value.h>
#include <readln.h>

#include "mystd.h"
#include "olf.h"
#include "mem.h"
#include "rng.h"
#include "func.h"
#include "dualplot.h"
#include "commands.h"
#include "calc.h"

using namespace std;


void my_gsl_error_handler (
    const char * reason, const char * file, int line, int gsl_errno )
{
    cout << ( "GSL error " + strg(gsl_errno) + ": " + reason + " in " +
              file + ", line " + strg(line) + "\n" );
    // throw does not work (probably because GSL is in C, not in C++)
}


int main()
{
    // ** Initializations **

    // External setup file:
    try {
        NRead::exemac("\\i " CONFDIR "/frida.ini");
    } catch ( string& s ) {
        cerr << s << endl;
    }

    // Math:
    gsl_set_error_handler( my_gsl_error_handler );
    NRNG::init();
    NFunctions::initialize();

    // Plot windows:
    NPloWin::initialize();

    // Time spent in frida, for optimizing numerical routines:
    clock_t clock_start = clock();
    clock_t clock_cmd_i;
    double time_cmd=0;


    // ** Main command loop **

    while(1) {

        try{
            // Get a word from the user:
            string prompt =
                (NOlm::FSel.size() ? NOlm::FSel.str() : "?") + " > ";
            string cmd = wask( prompt.c_str() );

            // If it's a list, then make it the default and request next word:
            if        ( cmd=="-" ) {
                NOlm::SelPop();
                continue;
            } else if ( cmd.find_first_of("*0123456789")==0 ) {
                bool islist = true;
                try {
                    CList newSel( cmd );
                } catch ( ... ) {
                    islist = false;
                }
                if ( islist ) {
                    CList newSel( cmd, 0, NOlm::MOM.size()-1 );
                    NOlm::SelPush( newSel );
                    continue;
                }
                // it's not a list, let's look what it is instead ...
            }

            // Regular commands:
            NOlm::FSelNew = CList();
            clock_cmd_i = clock();
			
            if ( cmd=="" ) {
                continue;
            } else if (cmd == "t") {
                double time_total =
                    ((double) (clock() - clock_start)) / CLOCKS_PER_SEC;
                printf( "total frida CPU time now %g\n", time_total );
                printf( "last command took %g\n", time_cmd );
                continue;

            } else if ( fridaCommand( cmd ) ) {
                // If there are new files, make them new default
                if( !NOlm::overwrite && NOlm::FSelNew.size()>0 )
                    NOlm::SelPush( NOlm::FSelNew );
                continue;
            }

            // Expression:
            NCalc::Calculator( cmd );

        } catch( string& s ) {
            cerr << s << endl;
            NRead::clear();
        } catch( const char* s ) {
            cerr << s << endl;
            NRead::clear();
        } catch( ... ) {
            cerr << "catched invalid exception\n";
            NRead::clear();
        }
        time_cmd = ((double) (clock() - clock_cmd_i)) / CLOCKS_PER_SEC;
    } // end main command loop
}
